/**
 * 
 */
package org.vamsas.client.simpleclient;

import java.io.File;

import org.vamsas.client.SimpleClient;

/**
 * @author jim Watches a particular file for its creation, deletion, or
 *         modification.
 */
public class FileWatcher {

  private File subject = null;

  private long lastStat[];
  boolean waslocked=false;
  boolean exists = false;

  /**
   * Make a watcher for a particular file. If the file doesn't exist, the
   * watcher will watch for its creation (and indicate a change of state) 
   * For locked files, the removal of a lock constitutes a change of 
   * state if the file was modified.
   * 
   * @param subject
   */
  private boolean checkLock() {
    if (subject!=null && subject.exists()) {
      Lock tl = new Lock(subject);
      if (tl.isLocked()) {
        tl.release();
        return false;
      }
      tl.release();
      return true;
    }
    return false;
  }
  
  private long[] getStat(File subject) {
    return new long[] { subject.lastModified(), subject.length() };
  }
  private boolean compStat(long[] stat, long[] newstat) {
    if (stat[0]!=newstat[0] || stat[1]!=newstat[1])
      return false;
    return true;
  }
  /**
   * Detect changes in file state and release of any
   * lock in place during change.
   * @return true if file state has changed
   */
  private boolean check() {
    if (subject != null) {
      if (!subject.exists()) {
        if (exists) {
          if (!waslocked) {
            // !checkLock()) {
          
            exists = false;
            // waslocked=false;
            return true;
          }
        }
        // locked - state change registered after lock is released
        return false;
      } else {
        long[] newStat = getStat(subject); // subject.lastModified();
        if (!checkLock()) {
          // file is free to access, return state change
          if (!exists || !compStat(lastStat, newStat)) {
            waslocked=false;
            exists=true;
            lastStat=newStat;
            return true;
          }
          // no change
          return false;
        } else {
          waslocked=true;
          return false;
        }
      }
    }
    return false;
  }
  public void setState() {
    if (subject!=null) {
      lastStat = getStat(subject);
      exists = subject.exists();
      waslocked = checkLock();
    }
  }
  
  public FileWatcher(File subject) {
    // TODO Auto-generated constructor stub
    this.subject = subject;
    setState();
  }

  public boolean hasChanged() {
    return check();
  }
}
